﻿using Dapper;
using Npgsql;
using System;
using System.Data;
using System.Linq;
using System.ServiceModel;
using System.Xml.Serialization;

namespace dapper12
{
    public class Item
    {
        [XmlElement]
        public int Id { get; set; }
        [XmlElement]
        public string Name { get; set; }
        [XmlElement]
        public string Description { get; set; }
        [XmlElement]
        public decimal Price { get; set; }
    }
    [ServiceContract]
    [XmlSerializerFormat]
    public interface IDapperService
    {
        [OperationContract]
        void Add(Item item);
        [OperationContract]
        void Remove(int id);
        [OperationContract]
        void Update(Item item);
        [OperationContract]
        Item FindByID(int id);
        [OperationContract]
        Item[] FindAll();
    }

    public class DapperService : IDapperService
    {
        private string connectionString = "User ID=postgres;Password=******;Host=localhost;Port=5433;Database=postgres;";
        internal IDbConnection Connection
        {
            get { return new NpgsqlConnection(connectionString); }
        }

        public void Add(Item item)
        {
            Console.WriteLine("Execute method Add");
            using (IDbConnection conn = Connection)
            {
                conn.Open();
                string command = "insert into \"Goods\"(\"Name\",\"Description\",\"Price\") ";
                command += "values(@Name,@Description,@Price)";
                conn.Execute(command, item);
            }
        }

        public Item[] FindAll()
        {
            Console.WriteLine("Execute method FindAll");
            using (IDbConnection conn = Connection)
            {
                conn.Open();
                var result = conn.Query<Item>("select * from \"Goods\"");
                return result.ToArray();
            }

        }

        public Item FindByID(int id)
        {
            Console.WriteLine("Execute method find");
            using (IDbConnection conn = Connection)
            {
                conn.Open();
                var res = conn.QueryFirstOrDefault<Item>("select * from \"Goods\" where \"Id\"=@Id LIMIT 1", new { Id = id });
                return res;
            }
        }

        public void Remove(int id)
        {
            Console.WriteLine("Execute method remove");
            using (IDbConnection conn = Connection)
            {
                conn.Open();
                conn.Execute("delete from \"Goods\" where id=@Id", id);
            }
        }

        public void Update(Item item)
        {
            Console.WriteLine("Execute method Update");
            using (IDbConnection conn = Connection)
            {
                conn.Open();
                string command = "update \"Goods\" set \"Name\"=@Name,\"Description\"=@Description,\"Price\"=@Price) ";
                command += "where \"Id\"=@Id";
                conn.Execute(command, item);
            }
        }
    }

    class Program
    {
        static void Main(string[] args)
        {
            ServiceHost host = new ServiceHost(typeof(DapperService), new Uri("http://localhost:6000/DapperService"));
            host.AddServiceEndpoint(typeof(IDapperService), new BasicHttpBinding(), "");
            host.Open();
            Console.WriteLine($"Start service {host.Description}, {host.State}");
            Console.ReadLine();
            host.Close();
            Console.WriteLine($"Close service {host.Description}, {host.State}");
            Console.ReadLine();
        }
    }
}
